@@ -77,6 +77,11 @@ History:
 2010-07-21 ROwen    Changed refreshAllVar to handle setting keyVars not current differently; instead of
                     implicitly basing it on the connection state, it now is based on a new argument.
                     Added readUnixTime field.
+2011-06-16 ROwen    API change: log messages receive a new keyword argument: cmdID.
+                    Added static method getMaxUserCmdID.
+                    Ditched obsolete "except (SystemExit, KeyboardInterrupt): raise" code
+2011-06-17 ROwen    Changed "type" to "msgType" in parsed message dictionaries to avoid conflict with builtin.
+2011-07-27 ROwen    Changed the executeCmd method to not log a message.
 """
 import sys
 import time
@@ -274,7 +279,7 @@ class KeyDispatcher(object):
           - cmdr: name of commander that triggered the message (string)
           - cmdID: command ID that triggered the message (int)
           - actor: the actor that generated the message (string)
-          - type: message type (character)
+          - msgType: message type (character)
           - data: dict of keyword: data_tuple entries;
             data_tuple is always a tuple, even if it contains one or zero values
         """
@@ -284,7 +289,7 @@ class KeyDispatcher(object):
         cmdr  = msgDict["cmdr"]
         cmdID   = msgDict["cmdID"]
         actor = msgDict["actor"]
-        msgType  = msgDict["type"]
+        msgType  = msgDict["msgType"]
         dataDict = msgDict["data"]
 
         # handle keywords
@@ -300,8 +305,6 @@ class KeyDispatcher(object):
             for keyVar in keyVarList:
                 try:
                     keyVar.set(valueTuple, msgDict = msgDict)
-                except (SystemExit, KeyboardInterrupt):
-                    raise
                 except:
                     traceback.print_exc(file=sys.stderr)
 
@@ -323,8 +326,6 @@ class KeyDispatcher(object):
         self.readUnixTime = time.time()
         try:
             msgDict = RO.ParseMsg.parseHubMsg(msgStr)
-        except (SystemExit, KeyboardInterrupt):
-            raise
         except Exception, e:
             self.logMsg(
                 msgStr = "CouldNotParse; Msg=%r; Text=%r" % (msgStr, RO.StringUtil.strFromException(e)),
@@ -338,8 +339,6 @@ class KeyDispatcher(object):
         # dispatch message
         try:
             self.dispatch(msgDict)
-        except (SystemExit, KeyboardInterrupt):
-            raise
         except Exception, e:
             sys.stderr.write("Could not dispatch: %r\n" % (msgDict,))
             traceback.print_exc(file=sys.stderr)
@@ -381,13 +380,12 @@ class KeyDispatcher(object):
         try:
             fullCmd = "%d %s %s" % (cmdVar.cmdID, cmdVar.actor, cmdVar.cmdStr)
             self.connection.writeLine (fullCmd)
-            self.logMsg (
-                msgStr = fullCmd,
-                actor = cmdVar.actor,
-            )
+#             self.logMsg (
+#                 msgStr = fullCmd,
+#                 actor = cmdVar.actor,
+#                 cmdID = cmdVar.cmdID,
+#             )
 #           print "executing:", fullCmd
-        except (SystemExit, KeyboardInterrupt):
-            raise
         except Exception, e:
             errMsgDict = self.makeMsgDict(
                 cmdID = cmdVar.cmdID,
@@ -395,12 +393,22 @@ class KeyDispatcher(object):
                     cmdVar.actor, cmdVar.cmdStr, RO.StringUtil.strFromException(e)),
             )
             self._replyCmdVar(cmdVar, errMsgDict)
+
+    @staticmethod
+    def getMaxUserCmdID():
+        """Return the maximum user command ID number.
+        
+        User command ID numbers range from 1 through getMaxUserCmdID()
+        Refresh command ID numbers range from getMaxUserCmdID() + 1 through 2 * getMaxUserCmdID()
+        """
+        return _CmdNumWrap
         
     def logMsg(self,
         msgStr,
         severity = RO.Constants.sevNormal,
         actor = "TUI",
         cmdr = None,
+        cmdID = 0,
     ):
         """Writes a message to the log.
         
@@ -423,9 +431,8 @@ class KeyDispatcher(object):
                 severity = severity,
                 actor = actor,
                 cmdr = cmdr,
+                cmdID = cmdID,
             )
-        except (SystemExit, KeyboardInterrupt):
-            raise
         except Exception, e:
             sys.stderr.write("Could not log: %r; severity=%r; actor=%r; cmdr=%r\n" % \
                 (msgStr, severity, actor, cmdr))
@@ -433,16 +440,15 @@ class KeyDispatcher(object):
     
     def logMsgDict(self, msgDict):
         try:
-            typeChar = msgDict["type"].lower()
-            severity = RO.KeyVariable.TypeDict[typeChar][1]
+            msgType = msgDict["msgType"].lower()
+            severity = RO.KeyVariable.TypeDict[msgType][1]
             self.logMsg(
                 msgStr = msgDict["msgStr"],
                 severity = severity,
                 actor = msgDict["actor"],
                 cmdr = msgDict["cmdr"],
+                cmdID = msgDict["cmdID"],
             )
-        except (SystemExit, KeyboardInterrupt):
-            raise
         except Exception, e:
             sys.stderr.write("Could not log message dict:\n%r\n" % (msgDict,))
             traceback.print_exc(file=sys.stderr)
@@ -451,7 +457,7 @@ class KeyDispatcher(object):
         cmdr = None,
         cmdID = 0,
         actor = None,
-        type = "f",
+        msgType = "f",
         dataStr = "",
     ):
         """Generate a hub message based on the supplied data.
@@ -466,13 +472,11 @@ class KeyDispatcher(object):
             cmdr,
             cmdID,
             actor,
-            type,
+            msgType,
         )
         msgStr = " ".join((headerStr, dataStr))
         try:
             return RO.ParseMsg.parseHubMsg(msgStr)
-        except (SystemExit, KeyboardInterrupt):
-            raise
         except Exception, e:
             sys.stderr.write("Could not make message dict from %r; error: %s" % (msgStr, e))
             traceback.print_exc(file=sys.stderr)
@@ -599,8 +603,6 @@ class KeyDispatcher(object):
                 # (thereby giving other time to other events)
                 # continuing where I left off
                 self._checkRemCmdID = self.tkWdg.after(1, self._checkRemCmdTimeouts, cmdVarIter)
-        except (SystemExit, KeyboardInterrupt):
-            raise
         except:
             sys.stderr.write ("RO.KeyDispatcher._checkRemCmdTimeouts failed\n")
             traceback.print_exc(file=sys.stderr)
@@ -630,7 +632,11 @@ class KeyDispatcher(object):
             keyVarNamesStr = ", ".join(sorted([kv.keyword for kv in keyVarSet]))
             errMsg = "Refresh command %s %s failed; keyVars not refreshed: %s" % \
                 (cmdVar.actor, cmdVar.cmdStr, keyVarNamesStr)
-            self.logMsg(errMsg, severity=RO.Constants.sevWarning)
+            self.logMsg(
+                msgStr = errMsg,
+                severity = RO.Constants.sevWarning,
+                cmdID = cmdVar.cmdID,
+            )
         elif keyVarSet:
             aKeyVar = iter(keyVarSet).next()
             actor = aKeyVar.actor
@@ -784,7 +790,7 @@ if __name__ == "__main__":
         "cmdr":"me",
         "cmdID":cmdID-1,
         "actor":"wrongActor",
-        "type":":",
+        "msgType":":",
         "data":dataDict,
     }
     print "\nDispatching message with wrong actor; nothing should happen"
@@ -794,7 +800,7 @@ if __name__ == "__main__":
         "cmdr":"me",
         "cmdID":cmdID-1,
         "actor":"test",
-        "type":":",
+        "msgType":":",
         "data":{},
     }
     print "\nDispatching message with wrong cmdID and no data; command callback should not be called"
@@ -804,7 +810,7 @@ if __name__ == "__main__":
         "cmdr":"me",
         "cmdID":cmdID,
         "actor":"wrongActor",
-        "type":":",
+        "msgType":":",
         "data":{},
     }
     print "\nDispatching message with wrong actor and no data; command callback should not be called"
@@ -814,7 +820,7 @@ if __name__ == "__main__":
         "cmdr":"me",
         "cmdID":cmdID,
         "actor":"test",
-        "type":":",
+        "msgType":":",
         "data":dataDict,
     }
     print "\nDispatching message correctly; all should work:"
