@@ -111,6 +111,8 @@ History:
 2009-07-20 ROwen    Modified for tweaked KeyDispatcher API.
                     Removed support for refreshTimeLim (it is now a constant in the KeyDispatcher).
 2011-02-17 ROwen    Document that addROWdgSet can take fewer widgets than values, but not more.
+2011-06-16 ROwen    Ditched obsolete "except (SystemExit, KeyboardInterrupt): raise" code
+2011-06-17 ROwen    Changed "type" to "msgType" in parsed message dictionaries to avoid conflict with builtin.
 """
 import sys
 import time
@@ -451,12 +453,12 @@ class KeyVar(RO.AddCallback.BaseMixin):
         Inputs:
         - valueList: a tuple of new values; if None then all values are reset to default
         - msgDict: the full keyword dictionary, see KeywordDispatcher for details
-          note: if supplied, msgDict must contain a field "type" with a valid type
+          note: if supplied, msgDict must contain a field "msgType" with a valid type character
 
         Errors:
         If valueList has the wrong number of elements then the data is rejected
         and an error message is printed to sys.stderr
-        If type in msgDict is missing or invalid, a warning message is printed
+        If msgType in msgDict is missing or invalid, a warning message is printed
         to sys.stderr and self.lastType is set to warning.
         """
         if valueList == None:
@@ -473,12 +475,12 @@ class KeyVar(RO.AddCallback.BaseMixin):
         self._msgDict = msgDict
         if msgDict:
             try:
-                self.lastType = msgDict["type"]
+                self.lastType = msgDict["msgType"]
             except KeyError:
-                sys.stderr.write("%s.set warning: 'type' missing in msgDict %r" % (self, msgDict))
+                sys.stderr.write("%s.set warning: 'msgType' missing in msgDict %r" % (self, msgDict))
                 self.lastType = "w"
             if not TypeDict.has_key(self.lastType):
-                sys.stderr.write("%s.set warning: invalid 'type'=%r in msgDict %r" % (self, self.lastType, msgDict))
+                sys.stderr.write("%s.set warning: invalid 'msgType'=%r in msgDict %r" % (self, self.lastType, msgDict))
                 self.lastType = "w"
 
         # print to stderr, if requested
@@ -894,14 +896,12 @@ class CmdVar(object):
             sys.stderr.write("Command %s already finished; no more replies allowed\n" % (self,))
             return
         self.lastReply = msgDict
-        msgType = msgDict["type"]
+        msgType = msgDict["msgType"]
         self.lastType = msgType
         for callTypes, callFunc in self.callTypesFuncList[:]:
             if msgType in callTypes:
                 try:
                     callFunc(msgType, msgDict, cmdVar=self)
-                except (SystemExit, KeyboardInterrupt):
-                    raise
                 except:
                     sys.stderr.write ("%s callback %s failed\n" % (self, callFunc))
                     traceback.print_exc(file=sys.stderr)
